<?php

namespace Drupal\micro_site\Plugin\EntityReferenceSelection;

use Drupal\user\Entity\User;
use Drupal\Core\Entity\Plugin\EntityReferenceSelection\DefaultSelection;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides entity reference selections for the site entity type.
 *
 * @EntityReferenceSelection(
 *   id = "default:site",
 *   label = @Translation("Site selection"),
 *   entity_types = {"site"},
 *   group = "default",
 *   weight = 1
 * )
 */
class SiteSelection extends DefaultSelection {

  /**
   * Sets the context for the alter hook.
   *
   * @var string
   */
  protected $field_type = 'editor';

  /**
   * {@inheritdoc}
   */
  public function buildEntityQuery($match = NULL, $match_operator = 'CONTAINS') {
    $query = parent::buildEntityQuery($match, $match_operator);
    $info = $query->getMetaData('entity_reference_selection_handler');
    // Let administrators do anything.
    if ($this->currentUser->hasPermission('administer site entities')) {
      return $query;
    }

    // Filter sites by the user's assignments, which are controlled by other
    // modules. Those modules must know what type of entity they are dealing
    // with, so look up the entity type and bundle.
    $info = $query->getMetaData('entity_reference_selection_handler');

    $context['field_type'] = $this->field_type;
    if (!empty($info->configuration['entity'])) {
      $context['entity_type'] = $info->configuration['entity']->getEntityTypeId();
      $context['bundle'] = $info->configuration['entity']->bundle();
    }

    // Load the current user.
    $account = User::load($this->currentUser->id());
    // Run the alter hook.
    $this->moduleHandler->alter('site_references', $query, $account, $context);

    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $selection_handler_settings = $this->configuration['handler_settings'];

    // Merge-in default values.
    $selection_handler_settings += array(
      // For the 'target_bundles' setting, a NULL value is equivalent to "allow
      // entities from any bundle to be referenced" and an empty array value is
      // equivalent to "no entities from any bundle can be referenced".
      'target_bundles' => NULL,
      'sort' => array(
        'field' => 'id',
        'direction' => 'ASC',
      ),
      'auto_create' => FALSE,
      'default_selection' => 'current',
    );

    $form['target_bundles'] = array(
      '#type' => 'value',
      '#value' => NULL,
    );

    $fields = array(
      'id' => $this->t('Site ID'),
      'name' => $this->t('Name'),
      'site_url' => $this->t('Site URL'),
    );

    $form['sort']['field'] = array(
      '#type' => 'select',
      '#title' => $this->t('Sort by'),
      '#options' => $fields,
      '#ajax' => FALSE,
      '#default_value' => $selection_handler_settings['sort']['field'],
    );

    $form['sort']['direction'] = array(
      '#type' => 'select',
      '#title' => $this->t('Sort direction'),
      '#required' => TRUE,
      '#options' => array(
        'ASC' => $this->t('Ascending'),
        'DESC' => $this->t('Descending'),
      ),
      '#default_value' => $selection_handler_settings['sort']['direction'],
    );

    return $form;
  }

}
